@@ -108,8 +108,8 @@ class SliceImageResult:
         self.image_dir = image_dir
 
     def add_sliced_image(self, sliced_image: SlicedImage):
-        assert isinstance(
-            sliced_image, SlicedImage
+        assert (
+            type(sliced_image) == SlicedImage
         ), "sliced_image must be a SlicedImage instance"
         self._sliced_image_list.append(sliced_image)
 
